#!/usr/bin/perl -w
#
# Sensor for gathering RPC performance data for Zabbix via
# nfsstat command.
# Copyright (c) 2009 Jakov Sosic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Changes and Modifications
# * Fri May  1 21:10:04 CEST 2009
# 	- rewritten sender to use only one connection
# 	- fixed alarms
# * Wed Sep  3 02:38:47 CEST 2008
# 	- changed 'nfstat -r' to 'nfstat -r -s'
# * Thu May 15 03:40:39 CEST 2008       
#       - script created

use strict;

# zabbix path
my $zabbix_sender="/usr/bin/zabbix_sender";
my $zabbix_confd="/etc/zabbix/zabbix_agentd.conf";
my $send_file="/var/tmp/zabbixSenderFileRPC";
my $zabbix_send_command="$zabbix_sender -c $zabbix_confd -i $send_file";
my $inputString = ''; 
my $hostname = `/bin/hostname -f`; 
chomp($hostname);

# signal handling
$SIG{'ALRM'} = sub {
	local $SIG{TERM} = 'IGNORE';
	kill TERM => -$$;
	print "0\n";
	exit(-1);
};

local $SIG{TERM} = sub {
	local $SIG{TERM} = 'IGNORE';
	kill TERM => -$$;
	print "0\n";
	exit(-1);
};

alarm 25;

# get nfs statistics
my $nfsstat = '/usr/sbin/nfsstat';
my @output = qx($nfsstat -r -s);

# if something brakes, exit
if ( $#output < 1 ) { print"0\n"; exit(-1); }

# store information to arrays
my $one_line;
my @keys;
my @values;
foreach $one_line (@output) {
   if ( $one_line =~ /calls/  ) { @keys = split (/[\s]+/,$one_line); }
   if ( $one_line =~ /[0-9]+/ ) { @values = split (/[\s]+/,$one_line); }
}

# send information to Zabbix server
my $count=0;
while ( $keys[$count] ){
   #print "$keys[$count] $values[$count]\n";
   #if ( qx($zabbix_send_command -k rpc.$keys[$count] -o $values[$count]) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
   $inputString .= $hostname ." rpc.". $keys[$count] ." ". $values[$count] ."\n" if ($values[$count]);
   $count++;
}

# write everything to file
open FH, ">", $send_file or die("Can not open file $send_file!");
print FH $inputString;
close(FH);

# finally, send the data
if ( qx($zabbix_send_command) =~ /Failed 0/ ) {
   unlink ($send_file) or die("Can not remove file $send_file!");
   print("1\n");
   exit(0);
} else {
   unlink ($send_file) or die("Can not remove file $send_file!");
   print("0\n");
   exit(-1);
}
#EOF
